      FUNCTION QUAD(RULE,N,F,A,B)
C
C
C   THIS  FUNCTION  SUBPROGRAM  APPROXIMATES  THE  VALUE  OF  AN
C   INTEGRAL I(F;A,B) OF A FUNCTION F(X) OVER THE INTERVAL (A,B)
C   USING THE N-POINT QUADRATURE RULE
C
C       Q(I;A,B)   =   W *F(X ) + W *F(X ) + ... + W *F(X )
C                       1    1     2    2           N    N
C
C   THE PARTICULAR RULE (DETERMINING THE  QUADRATURE  POINTS  XI
C   AND  THE  WEIGHTS  WI)  TO  BE  USED  IS  SPECIFIED  BY  THE
C   PARAMETERS RULE AND N.
C
C
C   CALLING SEQUENCE:  APPROX = QUAD(RULE,N,F,A,B)
C
C   PARAMETERS:
C
C   ON ENTRY:
C
C        RULE   - SUBROUTINE NAME
C                  A ROUTINE, RULE(N,A,B,P,W), FOR SUPPLYING THE
C                  QUADRATURE POINTS XI AND THE WEIGHTS WI RELA-
C                  TIVE TO THE INTERVAL  OF  INTEGRATION  (A,B).
C                  THE  FOLLOWING  VALUES  WILL ACCESS THE RULES
C                  THAT HAVE BEEN IMPLEMENTED IN THE PACKAGE:
C                    RULE = NCQ     - NEWTON-COTES RULES;
C                         = GLEGQ   - GAUSS-LEGENDRE RULES;
C                         = GLAGQ   - GAUSS-LAGUERRE RULES.
C                  SEE  THE  DOCUMENTATION  OF  THE  APPROPRIATE
C                  SUBROUTINE FOR DETAILS.
C                  IN ADDITION, A USER-SUPPLIED SUBROUTINE IMPL-
C                  EMENTATION OF  A QUADRATURE RULE CAN BE USED.
C                  SUCH A ROUTINE MUST,  OF COURSE,  CONFORM TO
C                  THE ABOVE CALLING SEQUENCE.
C
C        N      - INTEGER
C                  THE NUMBER OF QUADRATURE POINTS TO  BE  USED.
C                  THE LIMITS ON N WHEN RULE =
C                    NCQ   : 1 <= N <= 21;
C                    GLEGQ : 2 <= N <= 20;
C                    GLAGQ : 2 <= N <= 10.
C
C        F      - REAL
C                  A USER-SUPPLIED  FUNCTION  SUBPROGRAM,  F(X),
C                  FOR  EVALUATING  THE INTEGRAND AT ANY POINT X
C                  IN (A,B).
C
C        A,B    - REAL
C                  THE LEFT AND RIGHT ENDPOINTS OF THE  INTERVAL
C                  OF INTEGRATION.
C
C   ON RETURN:
C
C        QUAD   - REAL
C                  THE APPROXIMATE VALUE OF I(F;A,B).
C
C
C   SAMPLE CALLING PROGRAM:
C
C  C  *************************
C  C  *
C  C  *  THIS PROGRAM APPROXIMATES THE VALUE OF I(F;A,B) USING
C  C  *  A COMPOSITE 5-POINT GAUSS-LEGENDRE RULE.  THE NUMBER
C  C  *  OF SUBDIVISIONS OF (A,B) IS MDIV.
C  C  *
C  C  *************************
C  C
C        REAL A,B,H,XR,XL,APPROX
C        EXTERNAL F,GLEGQ
C        READ (5,*) A,B,MDIV
C        H = (B - A)/FLOAT(MDIV)
C        XR = A
C        APPROX = 0.E0
C        DO 10 IDIV = 1, MDIV
C             XL = XR
C             XR = A + FLOAT(IDIV)*H
C             APPROX = APPROX + QUAD(GLEGQ,5,F,XL,XR)
C  10    CONTINUE
C        WRITE (6,*) 'APPROX. VALUE OF THE INTEGRAL = ',APPROX
C        STOP
C        END
C
C        FUNCTION F(X)
C        F = ... EXPRESSION TO EVALUATE THE INTEGRAND
C        RETURN
C        END
C
C
      REAL P(20),W(20),F,A,B,SUM
C
C      ***************
C      *
C      * RETRIEVE  POINTS  AND  WEIGHTS,   TRANSFORMED  INTO THE
C      * SPECIFIED INTERVAL (A, B), FROM  THE  SUBROUTINE  RULE,
C      * AND APPLY THE QUADRATURE FORMULA.
C      *
C      ***************
C
         CALL RULE(N,A,B,P,W)
         SUM = 0.E0
         DO 10 I = 1,N
              SUM = SUM + W(I)*F(P(I))
  10     CONTINUE
         QUAD = SUM
      RETURN
      END
